package enverstest.type;

import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.hibernate.HibernateException;
import org.hibernate.engine.spi.SharedSessionContractImplementor;
import org.hibernate.type.LongType;
import org.hibernate.type.StringType;
import org.hibernate.type.Type;
import org.hibernate.usertype.CompositeUserType;

import enverstest.model.Value;

public class ValueType implements CompositeUserType 
{
    @Override
    public Class<?> returnedClass()
    {
        return Value.class;
    }

    @Override
    public boolean equals(Object arg0, Object arg1) throws HibernateException
    {
    	return (arg0 == null) ? arg1 == null : arg0.equals(arg1);
    }

    @Override
    public int hashCode(Object arg0) throws HibernateException
    {
    	return (arg0 == null) ? 0 : arg0.hashCode();
    }
   
    @Override
    public Object nullSafeGet(ResultSet resultset, String[] names, SharedSessionContractImplementor session, Object owner)
    throws HibernateException, SQLException
    {
        Long number = resultset.getLong(names[0]);
        if (resultset.wasNull())
        	number = null;
        
        String text = resultset.getString(names[1]);
        if (resultset.wasNull())
        	text = null;
        
        return new Value(text, number);
    }
    
    @Override
    public void nullSafeSet(PreparedStatement statement, Object value, int index, SharedSessionContractImplementor session)
    throws HibernateException, SQLException
    {
    	if (value == null) {
    		statement.setNull(index, LongType.INSTANCE.sqlType());
    		statement.setNull(index+1, StringType.INSTANCE.sqlType());
    	}
    	else {    	
    		Value val = (Value)value;
    		if (val.getNumber() == null)
        		statement.setNull(index, LongType.INSTANCE.sqlType());
    		else
    			statement.setLong(index, val.getNumber());
    		
    		if (val.getText() == null)
        		statement.setNull(index+1, StringType.INSTANCE.sqlType());
    		else
    			statement.setString(index+1, val.getText());
        }
    }
    
    @Override
    public String[] getPropertyNames()
    {
        return new String[] { "number", "text" };
    }

    @Override
    public Type[] getPropertyTypes()
    {
        return new Type[] { LongType.INSTANCE,
                            StringType.INSTANCE };
    }
    
    @Override
    public Object getPropertyValue(Object component, int property)
    {
        Value value = (Value)component;
        Object result = null;

        switch (property) {
        case 0:
            result = value.getNumber();
            break;

        case 1:
            result = value.getText();
        }

        return result;
    }

    @Override
    public void setPropertyValue (Object component, int property, Object value)
    {
        throw new UnsupportedOperationException("This class is immutable");
    }

    @Override
    public Object deepCopy(Object value)
    throws HibernateException
    {
    	// this object is already flat
    	return value;
    }

    @Override
    public boolean isMutable()
    {
        // This class is immutable
        return false;
    }
    
    @Override
    public Serializable disassemble(Object value, SharedSessionContractImplementor session)
    throws HibernateException
    {
    	return (Serializable)value;
    }

    @Override
    public Object assemble(Serializable cached, SharedSessionContractImplementor session, Object owner)
    throws HibernateException
    {
    	return cached;
    }

    @Override
    public Object replace(Object original, Object target, SharedSessionContractImplementor session, Object owner)
    throws HibernateException
    {
    	return original;
    }
}
