package enverstest.util;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class JpaUtil
{
    /** The log handle. */
    private static final Log LOG = LogFactory.getLog(JpaUtil.class);
    
    private static String persistenceUnit = null;
    private static EntityManagerFactory emf = null;
    private static EntityManager em = null;
    
    public static void init (final String pu)
    {
        if (persistenceUnit == null || !persistenceUnit.equals(pu)) {
            persistenceUnit = pu;
            emf = null;
            em = null;
        }
    }
    
    public static EntityManagerFactory getEntityManagerFactory()
    {
        if (emf == null)
            emf = Persistence.createEntityManagerFactory(persistenceUnit);
        
        return emf;
    }
    
    public static EntityManager getEntityManager()
    {
        if (em == null)
            em = getEntityManagerFactory().createEntityManager();
        
        return em;
    }
    
    public static EntityManager getEntityManager(final String pu)
    {
        init(pu);
        
        return getEntityManager();
    }
        
    public static EntityTransaction getTransaction()
    {
        return getEntityManager().getTransaction();
    }
    
    public static void beginTransaction()
    {
        try {
            getTransaction().begin();
       }
        catch (Exception ex) {
            LOG.error("beginTransaction() failed: " + ex);
        }
    }
    
    public static void commitTransaction()
    {
        try {
            getTransaction().commit();
        }
        catch (Exception ex) {
            LOG.error("commitTransaction() failed: " + ex);
        }
    }
    
    public static void rollbackTransaction() 
    {
        try {
            getTransaction().rollback();
        }
        catch (Exception ex) {
            LOG.error("rollbackTransaction() failed: " + ex);
        }
    }
    
    public static void flush()
    {
        getEntityManager().flush();
    }
    
    public static void shutdown() {
        LOG.debug("Shutting down EntityManager");
        // Close caches and connection pools
        getEntityManager().close();
        getEntityManagerFactory().close();

        // Clear static variables
        em = null;
        emf = null;
        persistenceUnit = null;
    }
}
